@@ -433,6 +433,7 @@ impl RetransmitStage {
         cluster_info: Arc<ClusterInfo>,
         retransmit_sockets: Arc<Vec<UdpSocket>>,
         repair_socket: Arc<UdpSocket>,
+        ancestor_hashes_socket: Arc<UdpSocket>,
         verified_receiver: Receiver<Vec<PacketBatch>>,
         exit: Arc<AtomicBool>,
         cluster_slots_update_receiver: ClusterSlotsUpdateReceiver,
@@ -486,6 +487,7 @@ impl RetransmitStage {
             verified_receiver,
             retransmit_sender,
             repair_socket,
+            ancestor_hashes_socket,
             exit,
             repair_info,
             leader_schedule_cache,
@@ -613,7 +615,7 @@ mod tests {
         let mut packet_batch = PacketBatch::new(vec![]);
         solana_streamer::packet::recv_from(&mut packet_batch, &me_retransmit, 1).unwrap();
         assert_eq!(packet_batch.packets.len(), 1);
-        assert!(!packet_batch.packets[0].meta.repair);
+        assert!(!packet_batch.packets[0].meta.repair());
     }
 
     #[test]
