@@ -311,8 +311,12 @@ impl Tvu {
         );
 
         let (cost_update_sender, cost_update_receiver) = channel();
-        let cost_update_service =
-            CostUpdateService::new(blockstore.clone(), cost_model.clone(), cost_update_receiver);
+        let cost_update_service = CostUpdateService::new(
+            exit.clone(),
+            blockstore.clone(),
+            cost_model.clone(),
+            cost_update_receiver,
+        );
 
         let (drop_bank_sender, drop_bank_receiver) = channel();
 
